
Work in progress: prototype of dots-signal-call-home.

The plan is to transform the current Incident Handler (which is essentially a subset of what call-home is intended for) into a DOTS Signal Call Home handler.


Description of current protocol
===============================

Currently, the incident handler works like this:

There is a server on the ISP side, which serves incident reports. These can be polled by the router ('give me all my incidents between these two timestamps), and have a custom format.

The format looks like this:

    {
      "incidents": [
        {
          "dst_addr": "178.18.82.80",
          "dst_port": 443,
          "id": 58,
          "incident_timestamp": 1524581768,
          "incident_type": "auto-generated for demo",
          "name": "demomalware",
          "report_timestamp": 1524581768,
          "severity": 3,
          "src_addr": "213.124.176.76",
          "src_port": 123
        }
      ]
    }

When handling incident reports, the router searches in it local traffic history for the offending device, and optionally blocks it.

There is also a signal option; the router listens on TCP port 40421. The server can send notifications to this listener, in the form of single time-stamps. Upon receiving a notification, the router fetches the above incident report and handles it.


DOTS SIGNAL CALL HOME
=====================

DOTS Signals (https://tools.ietf.org/html/draft-ietf-dots-signal-channel-34) contain a call-home signal: https://tools.ietf.org/html/draft-ietf-dots-signal-call-home-02.

We intend to gradually transform the above to this protocol, and then potentially make it 'fully DOTS compatible'.


APPROACH
========

For starters, we will change the format described earlier to the DOTS Signal Call Home format. An example of a message in that format is this:

NOTE: this is only the actial call-home part of the dots signal, the (future) DOTS signal processor must extract this section; in the first phase we only support this section.

    {
      "mitigation-scope": {
        "scope": [
          {
            "mitigation-id": 12332,
            "target-ip": [
              "2002:db8:6401::1",
              "2002:db8:6401::2"
            ],
            "target-port-range": [
              {
                "lower-port": 80,
                "upper-port": 80
              },
              {
                "lower-port": 443,
                "upper-port": 443
              },
              {
                "lower-port": 8080,
                "upper-port": 8080
              }
            ],
            "target-protocol": [
              6
            ]
          },
          {
            "mitigation-id": 12333,
            "target-ip": [
              "2002:db8:6402::1",
              "2002:db8:6402::2"
            ],
            "target-port-range": [
              {
                "lower-port": 82,
                "upper-port": 82
              },
              {
                "lower-port": 442,
                "upper-port": 442
              },
              {
                "lower-port": 8082,
                "upper-port": 8082
              }
            ],
            "target-protocol": [
              2
            ]
          }
        ]
      }
    }


Constructed example based on https://tools.ietf.org/html/draft-ietf-dots-signal-channel-34 and https://tools.ietf.org/html/draft-ietf-dots-signal-call-home-02:

notes: if only a lower port is given, it means 'just this port'.

    {
      "ietf-dots-signal-channel:mitigation-scope": {
        "scope": [
          {
            "target-prefix": [
              "2001:db8:6401::1/128",
              "2001:db8:6401::2/128"
            ],
            "target-port-range": [
              {
                "lower-port": 80
              },
              {
                "lower-port": 443
              },
              {
                "lower-port": 8080
              }
            ],
            "target-protocol": [
              6
            ],
            "lifetime": 3600,

            "source-prefix": [
              "2001:db8:6401::1/128",
              "2001:db8:6401::2/128"
            ],
            "source-port-range": [
              {
                "lower-port": 12345
              }
            ],
            "icmp-type-range": [
              {
                "lower-type": 0,
                "uppter-type": 8
              }
            ]
          }
        ]
      }
    }

This requires a number of internal modifications first:

1. No timestamps but mitigation-ids
2. Support for port ranges instead of actual ports (including empty ranges: probably meaning 'any port')
3. Support for traffic protocol type
4. Support for multiple target IPs
5. Support for (target) IP prefixes

Then we can update the data format:
1. Make the server prototype serve the DOTS signal format
2. Make the client handle and process that format

Improvement of internal handling:
3. Replace the current internal signaling through MQTT by an RPC call
4. Catch errors/status results

Add DOTS compatibility:
1. Listen to actual DOTS signals directly
2. Send errors/status results back to the 'dots client'




