# SPIN / Valibox release step checklist

### General setup
---------------------

There are 3 main components of the release environment:
- the SPIN source itself
- the sidn openwrt package repository
- the valibox-builder toolkit

The toolkit helps with running the openwrt build, collect additional info (such as the Changelog file), and can create the final images that are to be put on the website.

Shorthand used in this document for convenience:

* `/spin_src` the directory of the spin repository
* `/sidn_openwrt_pkgs` base directory of the package repository
* `/valibox_builder` base directory of the builder toolkit source
* `/openwrt` base directory of the openwrt build
For convenience, we also abbreviate calling the valibox-spin-builder/builder.py script with `vb`.


### Preparation depending on changes

#### Merge all relevant branches, and create a release branch for last-minute changes and fixes

Usually we work through beta releases first, so the branch can for instance be called `dev_beta_0.10`.

Merge all changes that weren't in the branch base to this branch, and make a note of the branch name for later.

#### Update global Changelog

The Changelog file that will be added to the release can be found in `valibox_builder/Valibox_Changelog.txt`. Update accordingly with changes.

#### Optional: update openWRT build config

In some cases, the OpenWRT .config may need to be updated. Since the openWRT build config differs per device target, we keep the changes from the 'base' config in `valibox_builder/devices/<device_name>/diffconfig'.

The builder will automatically apply these configs when building, and hard dependencies are added automatically, but if 'soft' dependencies are changed (say, for example, if you want to add the tcpdump package to the default build`), these need to be updated for every target.

For example, for the gl-inet build the procedure is as follows:

    cd openwrt
    cp /valibox_builder/devices/gl-ar150/diffconfig ./.config
    make menuconfig
    (apply changes and save config)
    ./scripts/feeds/diffconfig > /valibox_builder/devices/gl-ar150/diffconfig

Do this for all device targets that need to be changed, and commit the changes in the build directory.

#### Optional: add additional files to the device builds

Like .config files, each device target has their own `files/` directory tree for additional files (this is because a number of files need to be slightly different for different targets).

Make sure all `/valibox_builder/<device>/files/` trees contain the necessary files. Doublecheck that `/valibox_builder/<device>/files/etc/opgk/distfeeds.conf` contains the right urls.


### Build and create release images

Go to the parent directory of `/openwrt` and run `vb -e`. Important settings are `target_device = all`, `create_release = True` and the branches are correct. `make_arguments` should probably be empty (sometimes some openwrt packages fail to compile if for instance -j4 is added here).

If you have run builds before that had create_release set, it is prudent to remove the `valibox_release` subdirectory; this directory tree will contain the packages and image files that are to be copied to the website downloads/ directory, but older versions are not automatically removed by the build system.

Now run `vb -r`. This process will take quite some time, as it will need to (re)build the entire system for several architectures.

### Test release images

In valibox_release there are now release images for several devices. These should all be tested.

### Publish release

(note: this part should be done with someone who has administrator access on the website server).

Copy the valibox_release tree to the valibox website.

Update the `download_en.md` and `download_nl.md` pages. Make sure both the links and the sha256sums are correct, and that the changelog is updated here.

### Announce release

At the very minimum, the announcement should go to the mailinglist, and should contain the following parts:

- release number
- download links
- changelog (for this release, not necessarily the full historic one)
